"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const exts = require("./extensions");
exports.ELEMENT_NODE = 1;
exports.DOCUMENT_FRAGMENT_NODE = 11;
exports.DOCUMENT_NODE = 9;
exports.TEXT_NODE = 3;
exports.COMMENT_NODE = 8;
const attributes = utils_1.createMap();
attributes['style'] = applyStyle;
function isDocumentRoot(node) {
    return node.nodeType === 11 || node.nodeType === 9;
}
exports.isDocumentRoot = isDocumentRoot;
function isElement(node) {
    return node.nodeType === 1;
}
exports.isElement = isElement;
function isText(node) {
    return node.nodeType === 3;
}
exports.isText = isText;
function getAncestry(node, root) {
    const ancestry = [];
    let cur = node;
    while (cur !== root) {
        const n = cur;
        ancestry.push(n);
        cur = n.parentNode;
    }
    return ancestry;
}
exports.getAncestry = getAncestry;
const getRootNode = Node.prototype.getRootNode ||
    function () {
        let cur = this;
        let prev = cur;
        while (cur) {
            prev = cur;
            cur = cur.parentNode;
        }
        return prev;
    };
function reverseCollectNodeWithBreadth(parent, matcher, matches) {
    let cur = parent.lastChild;
    while (cur) {
        if (matcher(cur)) {
            matches.push(cur);
        }
        cur = cur.previousSibling;
    }
}
exports.reverseCollectNodeWithBreadth = reverseCollectNodeWithBreadth;
function reverseFindNodeWithBreadth(parent, matcher) {
    let cur = parent.lastChild;
    while (cur) {
        if (matcher(cur)) {
            return cur;
        }
        cur = cur.previousSibling;
    }
    return null;
}
exports.reverseFindNodeWithBreadth = reverseFindNodeWithBreadth;
function collectNodeWithBreadth(parent, matcher, matches) {
    let cur = parent.firstChild;
    if (matcher(cur)) {
        matches.push(cur);
    }
    while (cur) {
        if (matcher(cur.nextSibling)) {
            matches.push(cur);
        }
        cur = cur.nextSibling;
    }
}
exports.collectNodeWithBreadth = collectNodeWithBreadth;
function collectNodeWithDepth(parent, matcher, matches) {
    let cur = parent.firstChild;
    while (cur) {
        if (matcher(cur)) {
            matches.push(cur);
        }
        cur = cur.firstChild;
    }
}
exports.collectNodeWithDepth = collectNodeWithDepth;
function findNodeWithBreadth(parent, matcher) {
    let cur = parent.firstChild;
    while (cur) {
        if (matcher(cur)) {
            return cur;
        }
        cur = cur.nextSibling;
    }
    return null;
}
exports.findNodeWithBreadth = findNodeWithBreadth;
function findNodeWithDepth(parent, matcher) {
    let cur = parent.firstChild;
    while (cur) {
        if (matcher(cur)) {
            return cur;
        }
        cur = cur.firstChild;
    }
    return null;
}
exports.findNodeWithDepth = findNodeWithDepth;
function findDepthFirst(parent, matcher) {
    let cur = parent.firstChild;
    while (cur) {
        const found = findNodeWithDepth(cur, matcher);
        if (found) {
            return found;
        }
        cur = cur.nextSibling;
    }
    return null;
}
exports.findDepthFirst = findDepthFirst;
function collectDepthFirst(parent, matcher, matches) {
    let cur = parent.firstChild;
    while (cur) {
        collectNodeWithDepth(cur, matcher, matches);
        cur = cur.nextSibling;
    }
    return;
}
exports.collectDepthFirst = collectDepthFirst;
function findBreadthFirst(parent, matcher) {
    let cur = parent.firstChild;
    while (cur) {
        const found = findNodeWithBreadth(cur, matcher);
        if (found) {
            return found;
        }
        cur = cur.firstChild;
    }
    return null;
}
exports.findBreadthFirst = findBreadthFirst;
function applyChildNode(parent, fn) {
    let cur = parent.firstChild;
    while (cur) {
        fn(cur);
        cur = cur.nextSibling;
    }
}
exports.applyChildNode = applyChildNode;
function applyEachChildNode(parent, fn) {
    let cur = parent.firstChild;
    while (cur) {
        fn(cur);
        applyEachChildNode(cur, fn);
        cur = cur.nextSibling;
    }
}
exports.applyEachChildNode = applyEachChildNode;
function applyEachNode(parent, fn) {
    fn(parent);
    let cur = parent.firstChild;
    while (cur) {
        applyEachNode(cur, fn);
        cur = cur.nextSibling;
    }
}
exports.applyEachNode = applyEachNode;
function reverseApplyEachNode(parent, fn) {
    let cur = parent.lastChild;
    while (cur) {
        reverseApplyEachNode(cur, fn);
        fn(cur);
        cur = cur.previousSibling;
    }
    fn(parent);
}
exports.reverseApplyEachNode = reverseApplyEachNode;
function eachChildAndNode(node, fn) {
    const list = node.childNodes;
    for (let i = 0; i < list.length; i++) {
        fn(list[i]);
    }
    fn(node);
}
exports.eachChildAndNode = eachChildAndNode;
function nodeListToArray(items) {
    const list = [];
    for (let i = 0; i < items.length; i++) {
        list.push(items[i]);
    }
    return list;
}
exports.nodeListToArray = nodeListToArray;
function eachNodeAndChild(node, fn) {
    fn(node);
    const list = node.childNodes;
    for (let i = 0; i < list.length; i++) {
        fn(list[i]);
    }
}
exports.eachNodeAndChild = eachNodeAndChild;
function toHTML(node, shallow) {
    const div = document.createElement('div');
    div.appendChild(node.cloneNode(!shallow));
    return div.innerHTML;
}
exports.toHTML = toHTML;
function collectBreadthFirst(parent, matcher, matches) {
    let cur = parent.firstChild;
    while (cur) {
        collectNodeWithBreadth(cur, matcher, matches);
        cur = cur.firstChild;
    }
    return;
}
exports.collectBreadthFirst = collectBreadthFirst;
function getActiveElement(node) {
    const root = getRootNode.call(node);
    return isDocumentRoot(root) ? root.activeElement : null;
}
exports.getActiveElement = getActiveElement;
function getFocusedPath(node, root) {
    const activeElement = getActiveElement(node);
    if (!activeElement || !node.contains(activeElement)) {
        return [];
    }
    return getAncestry(activeElement, root);
}
exports.getFocusedPath = getFocusedPath;
function moveBefore(parentNode, node, referenceNode) {
    const insertReferenceNode = node.nextSibling;
    let cur = referenceNode;
    while (cur !== null && cur !== node) {
        const next = cur.nextSibling;
        parentNode.insertBefore(cur, insertReferenceNode);
        cur = next;
    }
}
exports.moveBefore = moveBefore;
function insertBefore(parentNode, node, referenceNode) {
    if (referenceNode === null) {
        parentNode.appendChild(node);
        return null;
    }
    parentNode.insertBefore(node, referenceNode);
    return null;
}
exports.insertBefore = insertBefore;
function replaceNode(parentNode, node, replacement) {
    if (replacement === null) {
        return null;
    }
    parentNode.replaceChild(replacement, node);
    return null;
}
exports.replaceNode = replaceNode;
function replaceNodeIf(targetNode, replacement) {
    if (replacement === null) {
        return false;
    }
    const parent = targetNode.parentNode;
    if (!parent) {
        return false;
    }
    parent.replaceChild(replacement, targetNode);
    return true;
}
exports.replaceNodeIf = replaceNodeIf;
function getNamespace(name) {
    if (name.lastIndexOf('xml:', 0) === 0) {
        return 'http://www.w3.org/XML/1998/namespace';
    }
    if (name.lastIndexOf('xlink:', 0) === 0) {
        return 'http://www.w3.org/1999/xlink';
    }
    return undefined;
}
exports.getNamespace = getNamespace;
function applyAttr(el, name, value) {
    if (value == null) {
        el.removeAttribute(name);
    }
    else {
        const attrNS = getNamespace(name);
        if (attrNS) {
            el.setAttributeNS(attrNS, name, String(value));
        }
        else {
            el.setAttribute(name, String(value));
        }
    }
}
exports.applyAttr = applyAttr;
function applyAttrs(el, values) {
    for (let key in values) {
        if (values[key] == null) {
            el.removeAttribute(key);
            continue;
        }
        el.setAttribute(key, values[key]);
    }
}
exports.applyAttrs = applyAttrs;
function applyProp(el, name, value) {
    el[name] = value;
}
exports.applyProp = applyProp;
function setStyleValue(style, prop, value) {
    if (prop.indexOf('-') >= 0) {
        style.setProperty(prop, value);
    }
    else {
        style[prop] = value;
    }
}
exports.setStyleValue = setStyleValue;
function applySVGStyle(el, name, style) {
    if (typeof style === 'string') {
        el.style.cssText = style;
    }
    else {
        el.style.cssText = '';
        const elStyle = el.style;
        for (const prop in style) {
            if (utils_1.has(style, prop)) {
                setStyleValue(elStyle, prop, style[prop]);
            }
        }
    }
}
exports.applySVGStyle = applySVGStyle;
function applyStyle(el, name, style) {
    if (typeof style === 'string') {
        el.style.cssText = style;
    }
    else {
        el.style.cssText = '';
        const elStyle = el.style;
        for (const prop in style) {
            if (utils_1.has(style, prop)) {
                setStyleValue(elStyle, prop, style[prop]);
            }
        }
    }
}
exports.applyStyle = applyStyle;
function applyStyles(el, style) {
    if (typeof style === 'string') {
        el.style.cssText = style;
    }
    else {
        el.style.cssText = '';
        const elStyle = el.style;
        for (const prop in style) {
            if (utils_1.has(style, prop)) {
                setStyleValue(elStyle, prop, style[prop]);
            }
        }
    }
}
exports.applyStyles = applyStyles;
function applySVGStyles(el, style) {
    if (typeof style === 'string') {
        el.style.cssText = style;
    }
    else {
        el.style.cssText = '';
        const elStyle = el.style;
        for (const prop in style) {
            if (utils_1.has(style, prop)) {
                setStyleValue(elStyle, prop, style[prop]);
            }
        }
    }
}
exports.applySVGStyles = applySVGStyles;
function applyAttributeTyped(el, name, value) {
    const type = typeof value;
    if (type === 'object' || type === 'function') {
        applyProp(el, name, value);
    }
    else {
        applyAttr(el, name, value);
    }
}
exports.applyAttributeTyped = applyAttributeTyped;
function getNamespaceForTag(tag, parent) {
    if (tag === 'svg') {
        return 'http://www.w3.org/2000/svg';
    }
    if (tag === 'math') {
        return 'http://www.w3.org/1998/Math/MathML';
    }
    if (parent == null) {
        return null;
    }
    return parent.namespaceURI;
}
exports.getNamespaceForTag = getNamespaceForTag;
function recordAttributes(node) {
    const attrs = {};
    const attributes = node.attributes;
    const length = attributes.length;
    if (!length) {
        return attrs;
    }
    for (let i = 0, j = 0; i < length; i += 1, j += 2) {
        const attr = attributes[i];
        attrs[attr.name] = attr.value;
    }
    return attrs;
}
exports.recordAttributes = recordAttributes;
function createElement(doc, nameOrCtor, key, content, attributes, namespace) {
    let el;
    if (typeof nameOrCtor === 'function') {
        el = new nameOrCtor();
        return el;
    }
    namespace = namespace.trim();
    if (namespace.length > 0) {
        switch (nameOrCtor) {
            case 'svg':
                el = doc.createElementNS('http://www.w3.org/2000/svg', nameOrCtor);
                break;
            case 'math':
                el = doc.createElementNS('http://www.w3.org/1998/Math/MathML', nameOrCtor);
                break;
            default:
                el = doc.createElementNS(namespace, nameOrCtor);
        }
    }
    else {
        el = doc.createElement(nameOrCtor);
    }
    el.setAttribute('_key', key);
    if (attributes) {
        applyAttrs(el, attributes);
    }
    if (content.length > 0) {
        el.innerHTML = content;
    }
    return el;
}
exports.createElement = createElement;
function createText(doc, text, key) {
    const node = doc.createTextNode(text);
    exts.Objects.PatchWith(node, 'key', key);
    return node;
}
exports.createText = createText;
function removeFromNode(fromNode, endNode) {
    const parentNode = fromNode.parentNode;
    let child = fromNode;
    while (child !== endNode) {
        const next = child.nextSibling;
        parentNode.removeChild(child);
        child = next;
    }
}
exports.removeFromNode = removeFromNode;
function fromBlob(o) {
    if (o === null || o === undefined) {
        return;
    }
    let data = null;
    const fileReader = new FileReader();
    fileReader.onloadend = function () {
        data = new Uint8Array(fileReader.result);
    };
    fileReader.readAsArrayBuffer(o);
    return data;
}
exports.fromBlob = fromBlob;
function fromFile(o) {
    if (o === null || o === undefined) {
        return;
    }
    let data = null;
    const fileReader = new FileReader();
    fileReader.onloadend = function () {
        data = new Uint8Array(fileReader.result);
    };
    fileReader.readAsArrayBuffer(o);
    return data;
}
exports.fromFile = fromFile;
function toInputSourceCapability(o) {
    if (o === null || o === undefined) {
        return;
    }
    return {
        FiresTouchEvent: o.firesTouchEvent,
    };
}
exports.toInputSourceCapability = toInputSourceCapability;
function toMotionData(o) {
    let md = { X: 0.0, Y: 0.0, Z: 0.0 };
    if (o === null || o === undefined) {
        return md;
    }
    md.X = o.x;
    md.Y = o.y;
    md.Z = o.z;
    return md;
}
exports.toMotionData = toMotionData;
function toRotationData(o) {
    if (o === null || o === undefined) {
        return;
    }
    const md = {};
    md.Alpha = o.alpha;
    md.Beta = o.beta;
    md.Gamma = o.gamma;
    return md;
}
exports.toRotationData = toRotationData;
function toMediaStream(o) {
    if (o === null || o === undefined) {
        return;
    }
    const stream = { Audios: [], Videos: [] };
    stream.Active = o.active;
    stream.Ended = o.ended;
    stream.ID = o.id;
    stream.Audios = [];
    stream.Videos = [];
    let audioTracks = o.getAudioTracks();
    if (audioTracks !== null && audioTracks !== undefined) {
        for (let i = 0; i < audioTracks.length; i++) {
            let track = audioTracks[i];
            let settings = track.getSettings();
            stream.Audios.push({
                Enabled: track.enabled,
                ID: track.id,
                Kind: track.kind,
                Label: track.label,
                Muted: track.muted,
                ReadyState: track.readyState,
                Remote: track.remote,
                AudioSettings: {
                    ChannelCount: settings.channelCount.Int(),
                    EchoCancellation: settings.echoCancellation,
                    Latency: settings.latency,
                    SampleRate: settings.sampleRate.Int64(),
                    SampleSize: settings.sampleSize.Int64(),
                    Volume: settings.volume,
                    MediaTrackSettings: {
                        DeviceID: settings.deviceId,
                        GroupID: settings.groupId,
                    },
                },
            });
        }
    }
    let videosTracks = o.getVideoTracks();
    if (videosTracks !== null && videosTracks !== undefined) {
        for (let i = 0; i < videosTracks.length; i++) {
            let track = videosTracks[i];
            let settings = track.getSettings();
            stream.Videos.push({
                Enabled: track.enabled,
                ID: track.id,
                Kind: track.kind,
                Label: track.label,
                Muted: track.muted,
                ReadyState: track.readyState,
                Remote: track.remote,
                VideoSettings: {
                    AspectRatio: settings.aspectRation,
                    FrameRate: settings.frameRate,
                    Height: settings.height.Int64(),
                    Width: settings.width.Int64(),
                    FacingMode: settings.facingMode,
                    MediaTrackSettings: {
                        DeviceID: settings.deviceId,
                        GroupID: settings.groupId,
                    },
                },
            });
        }
    }
    return stream;
}
exports.toMediaStream = toMediaStream;
function toTouches(o) {
    if (o === null || o === undefined) {
        return;
    }
    const touches = [];
    for (let i = 0; i < o.length; i++) {
        let ev = o.item(i);
        touches.push({
            ClientX: ev.clientX,
            ClientY: ev.clientY,
            OffsetX: ev.offsetX,
            OffsetY: ev.offsetY,
            PageX: ev.pageX,
            PageY: ev.pageY,
            ScreenX: ev.screenX,
            ScreenY: ev.screenY,
            Identifier: ev.identifier,
        });
    }
    return touches;
}
exports.toTouches = toTouches;
function toGamepad(o) {
    let pad = {};
    if (o === null || o === undefined) {
        return pad;
    }
    pad.DisplayID = o.displayId;
    pad.ID = o.id;
    pad.Index = o.index.Int();
    pad.Mapping = o.mapping;
    pad.Connected = o.connected;
    pad.Timestamp = o.timestamp;
    pad.Axes = [];
    pad.Buttons = [];
    let axes = o.axes;
    if (axes !== null && axes !== undefined) {
        for (let i = 0; i < axes.length; i++) {
            pad.Axes.push(axes[i]);
        }
    }
    let buttons = o.buttons;
    if (buttons !== null && buttons !== undefined) {
        for (let i = 0; i < buttons.length; i++) {
            const button = buttons[i];
            pad.Buttons.push({
                Value: button.value,
                Pressed: button.pressed,
            });
        }
    }
    return pad;
}
exports.toGamepad = toGamepad;
function toDataTransfer(o) {
    if (o === null || o === undefined) {
        return;
    }
    let dt = {};
    dt.DropEffect = o.dropEffect;
    dt.EffectAllowed = o.effectAllowed;
    dt.Types = o.types;
    dt.Items = [];
    const items = o.items;
    if (items !== null && items !== undefined) {
        for (let i = 0; i < items.length; i++) {
            const item = items.DataTransferItem(i);
            dt.Items.push({
                Name: item.name,
                Size: item.size.Int(),
                Data: fromFile(item),
            });
        }
    }
    dt.Files = [];
    const files = o.files;
    if (files !== null && files !== undefined) {
        for (let i = 0; i < files.length; i++) {
            const item = files[i];
            dFiles.push({
                Name: item.name,
                Size: item.size.Int(),
                Data: fromFile(item),
            });
        }
    }
    return dt;
}
exports.toDataTransfer = toDataTransfer;
function getMatchingNode(matchNode, matcher) {
    if (!matchNode) {
        return null;
    }
    if (matcher(matchNode)) {
        return matchNode;
    }
    while ((matchNode = matchNode.nextSibling)) {
        if (matcher(matchNode)) {
            return matchNode;
        }
    }
    return null;
}
exports.getMatchingNode = getMatchingNode;
//# sourceMappingURL=dom.js.map