"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const now = (function () {
    if (self.hasOwnProperty('performance')) {
        return ((performance.now ? performance.now.bind(performance) : null) ||
            (performance.mozNow ? performance.mozNow.bind(performance) : null) ||
            (performance.msNow ? performance.msNow.bind(performance) : null) ||
            (performance.oNow ? performance.oNow.bind(performance) : null) ||
            (performance.webkitNow ? performance.webkitNow.bind(performance) : null) ||
            Date.now.bind(Date));
    }
    return Date.now.bind(Date);
})();
const frameRate = 1000 / 60;
const vendors = ['ms', 'moz', 'webkit', 'o'];
function GetRAF() {
    let lastTime = 0;
    const mod = {};
    for (var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
        mod.requestAnimationFrame = window[vendors[x] + 'RequestAnimationFrame'];
        mod.cancelAnimationFrame =
            window[vendors[x] + 'CancelAnimationFrame'] || window[vendors[x] + 'RequestCancelAnimationFrame'];
    }
    if (!mod.requestAnimationFrame || !mod.cancelAnimationFrame)
        mod.requestAnimationFrame = function (callback, element) {
            const currTime = now();
            const timeToCall = Math.max(0, frameRate - (currTime - lastTime));
            const id = self.setTimeout(function () {
                try {
                    callback(currTime + timeToCall);
                }
                catch (e) {
                    console.log('Error: ', e);
                    setTimeout(function () {
                        throw e;
                    }, 0);
                }
            }, timeToCall);
            lastTime = currTime + timeToCall;
            return id;
        };
    if (!mod.cancelAnimationFrame) {
        mod.cancelAnimationFrame = function (id) {
            clearTimeout(id);
        };
    }
    return mod;
}
exports.GetRAF = GetRAF;
//# sourceMappingURL=raf-polyfill.js.map